"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AngularTestSuite = void 0;
const testing_1 = require("@angular/core/testing");
const testing_2 = require("@angular/platform-browser-dynamic/testing");
const suite_slimmer_1 = require("suite-slimmer");
const Document_mock_1 = require("./mocks/Document.mock");
const Location_mock_1 = require("./mocks/Location.mock");
const Window_mock_1 = require("./mocks/Window.mock");
class AngularTestSuite extends suite_slimmer_1.TestSuite {
    constructor(classType, testType, excludeOthers = false) {
        super(classType.name, excludeOthers);
        this.classType = classType;
        this.testType = testType;
    }
    addTest(description, callback, excludeOthers) {
        const callbackOverride = (classInstance, mocks) => callback(classInstance, mocks, this.fixture);
        return super.addTest(description, callbackOverride, excludeOthers);
    }
    async initializeTest(mockMapper, declarations, imports, providers) {
        switch (this.testType) {
            case 'component': {
                let componentFixture = testing_1.TestBed.createComponent(this.classType);
                let cls = componentFixture.componentInstance;
                this.fixture = componentFixture;
                return cls;
            }
            case 'service': {
                let cls = testing_1.TestBed.inject(this.classType);
                return cls;
            }
        }
    }
    async initializeTests(mockMapper, declarations, imports, providers) {
        const mocker = this.mockMapper.mocker;
        if (this.testType == 'component' && !declarations.includes(this.classType)) {
            declarations.push(this.classType);
        }
        if (!providers.find(p => p.provide === 'Document')) {
            const mock = mocker.mockService(Document_mock_1.DocumentMock);
            providers.push({ provide: 'Document', useValue: mock });
            mockMapper.addExplicit(Document, mock);
        }
        if (!providers.find(p => p.provide === 'Location')) {
            const mock = mocker.mockService(Location_mock_1.LocationMock);
            providers.push({ provide: 'Location', useValue: mock });
            mockMapper.addExplicit(Location, mock);
        }
        if (!providers.find(p => p.provide === 'Window')) {
            const mock = mocker.mockService(Window_mock_1.WindowMock);
            providers.push({ provide: 'Window', useValue: mock });
            mockMapper.addExplicit(Window, mock);
        }
        if ((0, testing_1.getTestBed)().platform == null) {
            await testing_1.TestBed.initTestEnvironment(testing_2.BrowserDynamicTestingModule, (0, testing_2.platformBrowserDynamicTesting)());
        }
        await testing_1.TestBed.resetTestingModule();
        await testing_1.TestBed.configureTestingModule({
            declarations,
            imports,
            providers
        }).compileComponents();
        // prevent Angular from resetting testing module
        this.resetTestingModule = testing_1.TestBed.resetTestingModule;
        testing_1.TestBed.resetTestingModule = () => testing_1.TestBed;
    }
    async disposeTests() {
        testing_1.TestBed.resetTestingModule = this.resetTestingModule;
    }
}
exports.AngularTestSuite = AngularTestSuite;
